<?php
/**
 * Created by IntelliJ IDEA.
 * User: alfmagne1
 * Date: 15/04/2017
 * Time: 13:35
 */

if(!defined("ABSPATH"))define( 'ABSPATH', dirname( __FILE__ ) . '/../wordpress/' );
require_once(ABSPATH . '/wp-config.php');
require_once("../wordpress/wp-includes/load.php");
require_once("../wordpress/wp-includes/plugin.php");
require_once("../wordpress/wp-includes/wp-db.php");
require_once(ABSPATH . WPINC . '/version.php');

require_once(__DIR__ . "/../../../autoload.php");


class MultiplayerTest  extends PHPUnit_Framework_TestCase{


    private $startFen = 'rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1';
    /**
     * @var wpdb $wpdb
     */
    private $wpdb;

    /**
     * @var DhtmlChessMultiplayerGameHandler $multiplayerGameHandler
     */
    private $multiplayerGameHandler;

    protected function setUp()
    {
        parent::setUp();
        DhtmlChessInstaller::enableTestMode();

        $this->wpdb = new wpdb("root", "", "wordpress3", "localhost");
        global $wpdb;
        $wpdb = $this->wpdb;

        $this->multiplayerGameHandler = new DhtmlChessMultiplayerGameHandler();

        $wpdb->query("delete from ". DhtmlChessDatabase::TABLE_ELO);
        $wpdb->query("delete from ". DhtmlChessDatabase::TABLE_MULTI_GAME);

    }

    public static function setUpBeforeClass()
    {
        parent::setUpBeforeClass(); // TODO: Change the autogenerated stub

        global $wpdb;
        $wpdb = new wpdb("root", "", "wordpress3", "localhost");
        $database = new DhtmlChessDatabase();
        $database->uninstall();

        DhtmlChessInstaller::disableTestMode();
        $database->install();
    }

    /**
     * @test
     */
    public function shouldBeAbleToCreateSeekObject(){
        // given
        $seek = new DhtmlChessMultiPlayerGame(1);

        // when
        $seek->fromElo(1000)->toElo(1600)->againstOpponent(2);

        $seekObj = $seek->getParams();

        // then
        $this->assertEquals(1000, $seekObj[DhtmlChessDatabase::COL_MIN_ELO]);
        $this->assertEquals(1600, $seekObj[DhtmlChessDatabase::COL_MAX_ELO]);
        $this->assertEquals(2, $seekObj[DhtmlChessDatabase::COL_SEEK_OPPONENT_ID]);
    }

    /**
     * @test
     */
    public function shouldBeAbleToCreateNewSeekInDatabase(){
        // given
        $seek = new DhtmlChessMultiPlayerGame(1, "alfmagne");
        $seek->fromElo(1000)->toElo(1600)->againstOpponent(2)->create();

        // when
        $seeks = $this->multiplayerGameHandler->getSeeks(1);

        // then
        $this->assertCount(1, $seeks);
    }

    /**
     * @test
     */
    public function shouldReturnParamObjectFromGame(){
        // given
        $seek = new DhtmlChessMultiPlayerGame(1, "alfmagne");
        $id = $seek->fromElo(1000)->toElo(1600)->againstOpponent(2)->create();

        // when
        $params = $this->multiplayerGameHandler->getGame($id);

        // then
        $this->assertInstanceOf(DhtmlChessMultiPlayerGame::class, $params);

        $this->assertEquals(1000, $params->val(DhtmlChessDatabase::COL_MIN_ELO));
        $this->assertEquals(1600, $params->val(DhtmlChessDatabase::COL_MAX_ELO));
        $this->assertEquals(2, $params->val(DhtmlChessDatabase::COL_SEEK_OPPONENT_ID));
        $this->assertEquals("r", $params->val(DhtmlChessDatabase::COL_SEEK_COLOR));
        $this->assertEquals(3, $params->val(DhtmlChessDatabase::COL_DAYS_PER_MOVE));
    }


    /**
     * @return DhtmlChessMultiPlayerGame
     */
    private function createGame(){

        $seek = new DhtmlChessMultiPlayerGame(1, "alfmagne");
        $id = $seek->fromElo(1000)->toElo(1600)->againstOpponent(2)->setSeekColor(DhtmlChessDatabase::COLOR_WHITE)->create();
        $game = $this->multiplayerGameHandler->getGame($id);
        $game->acceptSeek(2, "katrine")->save();
        return $game;
    }
    /**
     * @test
     */
    public function shouldCreateGameWhenSeekIsAccepted(){
        // given
        $seek = new DhtmlChessMultiPlayerGame(1, "alfmagne");
        $id = $seek->fromElo(1000)->toElo(1600)->againstOpponent(2)->setSeekColor(DhtmlChessDatabase::COLOR_WHITE)->create();

        // when
        $params = $this->multiplayerGameHandler->getGame($id);
        $params->acceptSeek(2, "katrine")->save();

        $params = $this->multiplayerGameHandler->getGame($id);

        // then
        $this->assertEquals(1, $params->val(DhtmlChessDatabase::COL_WHITE_ID));
        $this->assertEquals(2, $params->val(DhtmlChessDatabase::COL_BLACK_ID));
        $this->assertEquals("alfmagne", $params->val(DhtmlChessDatabase::COL_WHITE_NAME));
        $this->assertEquals("katrine", $params->val(DhtmlChessDatabase::COL_BLACK_NAME));
        $this->assertEquals($this->startFen, $params->val(DhtmlChessDatabase::COL_CURRENT_FEN));
        $this->assertEquals(1, $params->val(DhtmlChessDatabase::COL_USER_ID_TO_MOVE));
        $this->assertNotEmpty($params->val(DhtmlChessDatabase::COL_CREATED));
    }

    /**
     * @test
     */
    public function shouldBeAbleToAddMove(){
        // given
        $gameParams = $this->createGame();

        // when
        $gameParams->addMove("e2","e4", 1);
        $gameModel = json_decode($gameParams->getGame(), true);
        $firstMove = $gameModel["moves"][0];

        // then
        $this->assertCount(1, $gameModel["moves"]);
        $this->assertEquals("e4", $firstMove["m"]);
    }

    /**
     * @test
     */
    public function shouldUpdateTimestampOnMove(){
        // given
        $gameParams = $this->createGame();

        $tsStart = $gameParams->val(DhtmlChessDatabase::COL_CREATED);

        time_nanosleep(1, 0);
        // when
        $gameParams->addMove("e2","e4", 1)->save();


        $params = $this->multiplayerGameHandler->getGame($gameParams->id());

        // then
        $lastMoveTs = $params->val(DhtmlChessDatabase::COL_TS_LAST_MOVE);
        $createdTs = $params->val(DhtmlChessDatabase::COL_CREATED);


        $this->assertEquals($tsStart, $createdTs);
        $this->assertNotEmpty($lastMoveTs);

        $this->assertGreaterThan($createdTs, $lastMoveTs);
    }

    /**
     * @test
     */
    public function shouldUpdateUserIdToMoveOnMove(){
        $gameParams = $this->createGame();

        // when
        $gameParams->addMove("e2","e4", 1);

        // then
        $this->assertEquals(2, $gameParams->getUserIdToMove());
    }

    /**
     * @test
     */
    public function shouldCreateGameJSONWHenGameIsCreated(){
        // given
        $gameParams = $this->createGame();

        // when
        $array = $gameParams->getParams();

        // then
        $this->assertNotEmpty($array[DhtmlChessDatabase::COL_GAME]);
    }



    /**
     * @test
     */
    public function SeekShouldExpire(){

    }

    /**
     * @test
     */
    public function GameWillEndOnTimeOut(){

    }

    /**
     * @test
     */
    public function shouldEndGameOnCheckmate(){
        // given
        $gameParams = $this->createGame();

        // when
        $gameParams->addMove("f2","f3", 1);
        $gameParams->addMove("e7","e5", 2);
        $gameParams->addMove("g2","g4", 1);
        $gameParams->addMove("d8","h4", 2);

        $gameModel = json_decode($gameParams->getGame(), true);
        $this->assertCount(4, $gameModel["moves"]);

        // then
        $this->assertEquals(DhtmlChessDatabase::KEY_BLACK_WIN, $gameParams->getResult());
        $this->assertTrue($gameParams->isFinished());
    }


    /**
     * @test
     */
    public function shouldUpdateEloWhenGameEnds(){
        $game = $this->createGame();
        $game->addMove("f2","f3",1)->addMove("e7","e5",2)->addMove("g2","g4",1)->addMove("d8","h4", 2)->save();

        // when
        $elo = new DhtmlChessElo();
        $elo1 = $elo->getMultiPlayElo(1);
        $elo2 = $elo->getMultiPlayElo(2);

        // then
        $this->assertEquals(1370 - DhtmlChessElo::eloAdjustment(30), $elo1);
        $this->assertEquals(1430 + DhtmlChessElo::eloAdjustment(30), $elo2);
    }

    /**
     * @test
     */
    public function shouldBeAbleToClaimDraw(){


    }

    /**
     * @test
     */
    public function shouldBeAbleToDeleteSeek(){

    }

    /**
     * @test
     */
    public function shouldBeAbleToResign(){

    }


    /**
     * @test
     */
    public function shouldBeAbleToGetGameHistoryForAPlayer(){

    }

    /**
     * @test
     */
    public function shouldBeAbleToGetStatsForAPlayer(){


    }




}